@@ -2,6 +2,7 @@ from __future__ import unicode_literals, print_function
 import os
 import sys
 import six
+import stat
 import zlib
 import time
 import types
@@ -231,9 +232,8 @@ class Pipe(object):
             cmds.append(s.strip())
         return str(' | '.join(cmds))
 
-    @property
-    def stdout(self):
-        """standard output of the pipe. A file descriptor or an iteraror"""
+    def bg(self):
+        """Run processes in background. Return the last piped Popen object"""
         p = None
         self.processes = []
         self._stderr = None
@@ -249,6 +249,7 @@ class Pipe(object):
             elif isinstance(cmd, PyPipe):
                 cmd.stdin = p.stdout
                 stdin = cmd.iter_stdout
+                p = cmd
             else:
                 args = cmd.command_line(cmd.kwargs.get('shell', False))
 
@@ -270,7 +271,16 @@ class Pipe(object):
 
                 self.processes.append(p)
                 stdin = p.stdout
-        return stdin
+        return p
+
+    @property
+    def stdout(self):
+        """standard output of the pipe. A file descriptor or an iteraror"""
+        p = self.bg()
+        if isinstance(p, PyPipe):
+            return p.iter_stdout
+        else:
+            return p.stdout
 
     @classmethod
     def map(cls, args,
@@ -679,7 +689,6 @@ def wraps_module(mod):
 #####################
 
 
-@contextmanager
 def requires(*requirements, **kwargs):
     """Add extra dependencies in a virtualenv"""
     if '/.tox/' in sys.executable:
@@ -700,13 +709,13 @@ def requires(*requirements, **kwargs):
     if not test.d(venv):
         import urllib
         url = 'https://raw.github.com/pypa/virtualenv/master/virtualenv.py'
-        urllib.urlretrieve(url, '/tmp/virtualenv.py')
+        urllib.urlretrieve(url, '/tmp/_virtualenv.py')
         sh[sys.executable]('-S /tmp/_virtualenv.py', venv) > 1
         sh.rm('/tmp/_virtualenv*', shell=True)
         print('Installing %s...' % ', '.join(requirements))
         sh.pip('install -qM', *requirements) > 1
     elif env.chut_virtualenv:
-        upgrade = '--upgrade-deps' in sys.argv
+        upgrade = '--upgrade' in sys.argv
         if (env.chut_upgrade or upgrade):
             installed = ''
         else:
@@ -718,9 +727,7 @@ def requires(*requirements, **kwargs):
     executable = os.path.join(bin_dir, 'python')
     if not env.chut_virtualenv:
         env.chut_virtualenv = venv
-        env.CHUTIFIED_FILES = os.environ.get('CHUTIFIED_FILES', '')
         os.execve(executable, [executable] + sys.argv, env)
-    yield True
 
 
 class console_script(object):
@@ -778,7 +785,6 @@ class Generator(object):
     docopts like arguments"""
 
     _modules = {}
-    _env_key = str('CHUTIFIED_FILES')
 
     def __init__(self, **args):
         self.devel = args.get('--devel') or args.get('devel')
@@ -823,37 +829,39 @@ class Generator(object):
         return modules
 
     def generate(self, filename, args=None, **kwargs):
-        chutified = os.environ.get(self._env_key, str(''))
-        if filename in chutified:
-            return []
         if args is None:
             args = {}
         args.update(kwargs)
         dirname = os.path.dirname(filename)
-        sys.path.insert(0, dirname)
-        name = inspect.getmodulename(filename)
+        mod_name = inspect.getmodulename(filename)
         os.environ.update(env)
-        mod = __import__(name, {'os': os}, {'os': os})
-        os.environ[self._env_key] = str(chutified + filename + ':')
 
-        console_scripts = []
-        for k, v in mod.__dict__.items():
-            if getattr(v, '_console_script', False) is True:
-                console_scripts.append(k)
+        console_scripts = list(sh.grep('-A4 -E @.*console_script', filename))
+        console_scripts = [s for s in console_scripts if s.startswith('def ')]
 
         scripts = []
-        for name in console_scripts:
+        loop = '--loop' in sys.argv or '-l' in sys.argv
+        while not os.path.isfile(filename):
+            time.sleep(.1)
+        mtime = os.stat(filename)[stat.ST_MTIME]
+        for func_name in sorted(set(console_scripts)):
+            name = func_name[4:].split('(')[0]
             script = os.path.join(self.dest, name.replace('_', '-'))
+            if os.path.isfile(script) and loop:
+                smtime = os.stat(script)[stat.ST_MTIME]
+                if mtime <= smtime:
+                    continue
             with open(script, 'w') as fd:
                 fd.write(SCRIPT_HEADER % self.args + self.mods + LOAD_MODULES)
                 if self.devel:
                     fd.write('sys.path.insert(0, "%s")\n' % dirname)
-                    fd.write('import %s\n' % mod.__name__)
+                    fd.write('import %s\n' % mod_name)
                     fd.write('if __name__ == "__main__":\n')
-                    fd.write('    %s.%s()\n' % (mod.__name__, name))
+                    fd.write('    %s.%s()\n' % (mod_name, name))
                 else:
-                    fd.write(inspect.getsource(mod).replace('__main__',
-                                                            '__chutified__'))
+                    with open(filename) as mod:
+                        fd.write(mod.read().replace('__main__',
+                                                    '__chutified__'))
                     fd.write("\nif __name__ == '__main__':\n    %s()\n" % name)
             executable = sh.chmod('+x', script)
             if executable:
@@ -877,6 +885,8 @@ class Generator(object):
 
 SCRIPT_HEADER = '''
 #!/usr/bin/env %(interpreter)s
+# This script is generated with chut. Do NOT edit this file.
+# All your changes will be lost at the next generation.
 
 version = %(version)s
 
