@@ -50,7 +50,7 @@ def translate(args):
         print('You must install casperjs first')
         return 1
     env.tr_pair = args['--langs'].replace(':', '|')
-    script = str(mktemp('translate-XXXX.js'))
+    script = str(mktemp('--tmpdir translate-XXXX.js'))
     atexit.register(rm(script))
     stdin(SCRIPT) > script
 
